@@ -163,7 +163,7 @@
    wm-size-hints-width-inc wm-size-hints-win-gravity wm-size-hints-x
    wm-size-hints-y wm-zoom-hints write-bitmap-file write-resources xatom
    x-error
-
+   keysym-name
    trace-display suspend-display-tracing resume-display-tracing
    untrace-display show-trace
    display-trace ; for backwards compatibility describe-request describe-event describe-reply
@@ -361,7 +361,7 @@
 (defclass pixmap      (drawable)              ())
 (defclass cursor      (xid-object)            ())
 (defclass colormap    (xid-object)            (#|(visual-info :initarg :visual-info :accessor colormap-visual-info)|#))
-(defclass gcontext    (ptr-object)            ((%dashes) (%clip-mask)))
+(defclass gcontext    (ptr-object)            ((%dashes) (%clip-mask) (%timestamp :accessor gcontext-internal-timestamp :initform 0)))
 (defclass screen      (ptr-object)            ())
 (defclass font        (xid-object)
   ((font-info :initform nil :initarg :font-info)
@@ -373,6 +373,7 @@
 ;;;; --------------------------------------------------------------------------
 ;;;;  Setf Methods
 ;;;; --------------------------------------------------------------------------
+(defsetf ACCESS-CONTROL               SET-ACCESS-CONTROL)
 (defsetf CLOSE-DOWN-MODE (display) (mode)
   `(SET-CLOSE-DOWN-MODE ,mode ,display))
 (defsetf DISPLAY-AFTER-FUNCTION       SET-DISPLAY-AFTER-FUNCTION)
@@ -447,49 +448,48 @@
                       &body clauses)
   (let ((slots (gensym)))
     ;; FIXME this implementation is not 100%
-    force-output-p
     `(process-event ,display
-                    :timeout ,timeout
-                    :peek-p ,peek-p
-                    :discard-p ,discard-p
-                    :force-output-p ,force-output-p
+       :timeout ,timeout
+       :peek-p ,peek-p
+       :discard-p ,discard-p
+       :force-output-p ,force-output-p
        :handler
-       #'(lambda (&rest ,slots &key event-key &allow-other-keys)
-           ;; (print slots)
-           (cond ,@(mapcar
-                    #'(lambda (clause)
-                        (let ((event-or-events (car clause))
-                              (binding-list (cadr clause))
-                              (test-form (caddr clause))
-                              (body-forms (cdddr clause)))
-                          (cond ((member event-or-events '(t otherwise))
-                                 ;;Special case
-                                 `((and t
-                                        ,@(if test-form
-                                              (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys) ,test-form) ,slots))
-                                            nil))
-                                   ,@(if body-forms
-                                         (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys)
-                                                           ,@body-forms)
-                                                       ,slots))
-                                       nil)) )
-                                (t
-                                 ;; Make-up keywords from the event-keys
-                                 (unless (listp event-or-events)
-                                   (setq event-or-events (list event-or-events)))
-                                 (setq event-or-events (mapcar #'kintern event-or-events))
-                                 `((and ,(if (cdr event-or-events)
-                                             `(member event-key ',event-or-events)
-                                           `(eq event-key ',(car event-or-events)))
-                                        ,@(if test-form
-                                              (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys) ,test-form) ,slots))
-                                            nil))
-                                   ,@(if body-forms
-                                         (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys)
-                                                           ,@body-forms)
-                                                       ,slots))
-                                       nil) )))))
-                    clauses) )) )))
+       (lambda (&rest ,slots &key event-key &allow-other-keys)
+         ;; (print slots)
+         (cond ,@(mapcar
+                  (lambda (clause)
+                    (let ((event-or-events (car clause))
+                          (binding-list (cadr clause))
+                          (test-form (caddr clause))
+                          (body-forms (cdddr clause)))
+                      (cond ((member event-or-events '(t otherwise))
+                             ;;Special case
+                             `((and t
+                                    ,@(if test-form
+                                          (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys) ,test-form) ,slots))
+                                          nil))
+                               ,@(if body-forms
+                                     (list `(apply (lambda (&key ,@binding-list &allow-other-keys)
+                                                     ,@body-forms)
+                                                   ,slots))
+                                     nil)))
+                            (t ;; Make-up keywords from the event-keys
+                             (unless (listp event-or-events)
+                               (setq event-or-events (list event-or-events)))
+                             (setq event-or-events
+                                   (mapcar #'kintern event-or-events))
+                             `((and ,(if (cdr event-or-events)
+                                         `(member event-key ',event-or-events)
+                                         `(eq event-key ',(car event-or-events)))
+                                    ,@(if test-form
+                                          (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys) ,test-form) ,slots))
+                                          nil))
+                               ,@(if body-forms
+                                     (list `(apply #'(lambda (&key ,@binding-list &allow-other-keys)
+                                                       ,@body-forms)
+                                                   ,slots))
+                                     nil))))))
+                  clauses))))))
 
 (defmacro EVENT-CASE ((&rest args) &body clauses)
   ;; Event-case is just event-cond with the whole body in the test-form
@@ -1323,8 +1323,7 @@
   (make-instance 'gcontext))
 
 ;; What has that to do with graphics?!
-(defparameter *keyword-package* (find-package :keyword))
-(defun kintern (name) (intern (string name) *keyword-package*))
+(defun kintern (name) (intern (string name) #,(find-package :keyword)))
 
 ;;;;From depdefs.lisp
 ;;;;
@@ -1484,13 +1483,15 @@
 
 ;;; Error handler, we probably want a proper condition hierarchy, but for a first approach this may be enough:
 
-(defun default-error-handler (display error-code &key current-sequence sequence major minor resource-id atom-id value)
-  (cerror "Just ignore this error and proceed."
-          "Asynchronous ~A on display ~S, sequence ~D.~%Opcode is ~D.~D. Current sequence is ~D. ~A"
+(defun default-error-handler (display error-code &key current-sequence sequence
+                              major minor resource-id atom-id value)
+  (cerror "Ignore this error and proceed."
+          "Asynchronous ~A on display ~S, sequence ~D. Opcode is ~D.~D. Current sequence is ~D.~A"
           error-code display sequence major minor current-sequence
-          (cond (resource-id (format nil "Resource ID is #x~8,'0x." resource-id))
-                (value (format nil "Bad value is ~D" value))
-                (atom-id (format nil "Bad atom ID is #x~8,'0x." atom-id))
+          ;; at most one of RESOURCE-ID, ATOM-ID, and VALUE is available
+          (cond (resource-id (format nil " Resource ID is #x~8,'0x." resource-id))
+                (value (format nil " Bad value is ~D." value))
+                (atom-id (format nil " Bad atom ID is #x~8,'0x." atom-id))
                 (t ""))))
 
 (define-condition x-error (error)
@@ -1535,3 +1536,20 @@
 
 (pushnew "XLIB" custom:*system-package-list* :test #'string=)
 (pushnew "XPM" custom:*system-package-list* :test #'string=)
+
+;; some functions are not implemented:
+(macrolet ((undefined (name)
+             `(define-compiler-macro ,name (&whole form &rest args)
+                (declare (ignore args))
+                (progn (warn "~S is not implemented: ~S" ',name form)
+                       form))))
+(undefined DISPLAY-RESOURCE-ID-BASE)
+(undefined DISPLAY-RESOURCE-ID-MASK)
+(undefined DISPLAY-TRACE)
+(undefined DRAW-GLYPH)
+(undefined DRAW-IMAGE-GLYPH)
+(undefined TRANSLATE-DEFAULT)
+(undefined QUEUE-EVENT)
+(undefined CHANGE-KEYBOARD-MAPPING)
+(undefined KEYBOARD-MAPPING)
+)
