@@ -1,13 +1,13 @@
 ;; Module for Raw Sockets / CLISP
 ;; Fred Cohen, 2003-2004
 ;; Don Cohen, 2003-2004
-;; Sam Steingold 2004-2005
+;; Sam Steingold 2004-2006
 ;; <http://www.opengroup.org/onlinepubs/007908799/xns/syssocket.h.html>
 
 (defpackage #:rawsock
   (:documentation "Raw Socket access")
   (:use #:lisp)
-  (:shadowing-import-from "EXPORTING" #:defun #:defstruct)
+  (:shadowing-import-from "EXPORTING" #:defun #:defstruct #:define-condition)
   (:export #:buffer #:resize-buffer #:accept #:bind #:connect
            #:getpeername #:getsockname #:protocol #:network #:message
            #:sock-listen #:recv #:recvfrom #:recvmsg
@@ -67,7 +67,8 @@
          (address (make-sockaddr :AF_UNIX
                                  (ext:convert-string-to-bytes
                                   (namestring (ext:absolute-pathname pathname))
-                                  ext:*pathname-encoding*))))
+                                  #+UNICODE custom:*pathname-encoding*
+                                  #-UNICODE :default))))
     (connect socket address)
     (values socket address)))
 
@@ -94,3 +95,9 @@ Passes :TYPE to SOCKET and all the other options to MAKE-STREAM."
   (when (fboundp 'rawsock:getnameinfo)
     (multiple-value-bind (node service) (rawsock:getnameinfo addr)
       (format out "sockaddr node: ~S, service: ~S~%" node service))))
+
+(define-condition eai (error)
+  (($ecode :reader eai-code :initarg :code)
+   ($message :reader eai-message :initarg :message))
+  (:documentation "getaddrinfo()/getnameinfo() error, see <netdb.h>")
+  (:report (lambda (c out) (format out "[~S]: ~A" (eai-code c) (eai-message c)))))
