@@ -134,6 +134,8 @@ to print the corresponding values, or T for all of them.")
        (format stream (TEXT "a load-time evaluation promise.")))
       (EXT:SYMBOL-MACRO
        (format stream (TEXT "a symbol macro handler.")))
+      (EXT:GLOBAL-SYMBOL-MACRO
+       (format stream (TEXT "a global symbol macro handler.")))
       (SYS::MACRO
        (format stream (TEXT "a macro expander."))
        (terpri stream)
@@ -213,6 +215,10 @@ to print the corresponding values, or T for all of them.")
     (format stream (TEXT "an instance of the CLOS class ~S.")
             (clos:class-of obj))
     (describe-slotted-object obj stream))
+  (:method ((obj clos:funcallable-standard-object) (stream stream))
+    (format stream (TEXT "an instance of the CLOS class ~S, can be used as a function.")
+            (clos:class-of obj))
+    (describe-slotted-object obj stream))
   (:method ((obj structure-object) (stream stream)) ; CLISP specific
     (format stream (TEXT "a structure of type ~S.")
             (type-of obj))
@@ -506,7 +512,7 @@ to print the corresponding values, or T for all of them.")
                             (clos:method-specializers meth)))))
         (format stream (TEXT "No methods.")))))
   (:method ((obj function) (stream stream))
-    (ecase (type-of obj)
+    (ecase (type-of obj)        ; not etypecase!
       #+FFI
       (FFI::FOREIGN-FUNCTION
        (format stream (TEXT "a foreign function of foreign type ~S.")
@@ -538,6 +544,8 @@ to print the corresponding values, or T for all of them.")
                            (TEXT "For more information, evaluate ~{~S~^ or ~}.")
                            `((DISASSEMBLE ,funform))))))))))
       (FUNCTION
+       ;; we do not use ETYPECASE here to ensure that if we do get here,
+       ;; we are dealing with an Iclosure object
        (format stream (TEXT "an interpreted function."))
        (let ((doc (sys::%record-ref obj 2)))
          (terpri stream)
