@@ -1,4 +1,5 @@
 ;;; Gray streams, following David N. Gray's STREAM-DEFINITION-BY-USER proposal
+;;; ftp://parcftp.xerox.com/pub/cl/cleanup/mail/stream-definition-by-user.mail
 
 (in-package "GRAY")
 (common-lisp:export
@@ -53,10 +54,14 @@
 ;; Classes
 
 (eval-when (compile load eval)
+  (defmethod clos:validate-superclass ((class class) (superclass (eql clos::<stream>)))
+    (or (call-next-method)
+        (eq (clos:class-name class) 'fundamental-stream)))
   (let ((clos::*allow-mixing-metaclasses* t))
     (clos:defclass fundamental-stream (stream clos:standard-object)
       (($open :type boolean :initform t) ; whether the stream is open
        ($reval :type boolean :initform nil) ; whether read-eval is allowed
+       ($penl :type boolean :initform nil) ; whether an elastic newline is pending
 ) ) ) )
 
 (clos:defclass fundamental-input-stream (fundamental-stream)
@@ -84,8 +89,8 @@
 )
 
 (clos:defclass fundamental-character-input-stream (fundamental-input-stream fundamental-character-stream)
-  ()
-)
+  (($lastchar :initform nil) ; last character read (and not yet unread)
+) )
 
 (clos:defclass fundamental-character-output-stream (fundamental-output-stream fundamental-character-stream)
   ()
@@ -106,7 +111,8 @@
     (apply #'sys::built-in-stream-close stream args))
   (:method ((stream fundamental-stream) &rest more-args)
     (declare (ignore more-args))
-    (clos:with-slots ($open) stream
+    (clos:with-slots ($open $penl) stream
+      (when $penl (setq $penl nil) (write-char #\Newline stream))
       (prog1 $open (setq $open nil))
   ) )
 )
@@ -315,17 +321,18 @@
   ) )
 )
 
-(clos:defgeneric stream-read-byte-sequence
-    (stream sequence &optional start end no-hang)
+(clos:defgeneric stream-read-byte-sequence (stream sequence
+                                            &optional start end no-hang interactive)
   (:method ((stream fundamental-input-stream) (sequence vector)
-            &optional (start 0) (end nil) (no-hang nil))
+            &optional (start 0) (end nil) (no-hang nil) (interactive nil))
     ;; sequence is a (simple-array (unsigned-byte 8) (*)),
     ;; and start and end are suitable integers.
     (unless end (setq end (length sequence)))
     (do ((index start (1+ index)))
         ((eql index end) index)
-      (let ((x (if no-hang (stream-read-byte-no-hang stream)
-                   (stream-read-byte stream))))
+      (let ((x (if (or no-hang (and interactive (> index start)))
+                 (stream-read-byte-no-hang stream)
+                 (stream-read-byte stream))))
         (when (or (null x) (eq x ':EOF)) (return index))
         (setf (aref sequence index) x)))))
 
@@ -334,9 +341,9 @@
 (clos:defgeneric stream-write-byte (stream integer))
 
 (clos:defgeneric stream-write-byte-sequence (stream sequence
-                                             &optional start end no-hang)
+                                             &optional start end no-hang interactive)
   (:method ((stream fundamental-output-stream) (sequence vector)
-            &optional (start 0) (end nil) (no-hang nil))
+            &optional (start 0) (end nil) (no-hang nil) (interactive nil))
     ;; sequence is a (simple-array (unsigned-byte 8) (*)),
     ;; and start and end are suitable integers.
     ;; if no-hang and you write less than end-start bytes then you should
@@ -345,6 +352,9 @@
     (when no-hang
       (error "~S: ~S is not supported by the default method"
              'stream-write-byte-sequence :NO-HANG))
+    (when interactive
+      (error "~S: ~S is not supported by the default method"
+             'stream-write-byte-sequence :INTERACTIVE))
     (unless end (setq end (length sequence)))
     (do ((index start (1+ index)))
         ((eql index end) nil)
